;;;
;;; Copyright (C) 2005 by Robert James Kaes <rjk@wormbytes.ca>
;;;   All rights reserved.
;;;

;;; Stock files

(require 'remember)
(require 'planner)

(require 'remember-planner)
(require 'planner-schedule)
(require 'planner-timeclock)
(require 'planner-timeclock-summary)
(require 'planner-deadline)
(require 'planner-appt)
(require 'planner-cyclic)
(require 'planner-diary)


;;; Keybindings

(planner-install-extra-context-keybindings)

(define-key planner-mode-map "\C-c\C-n" 'planner-create-note-from-task)
(define-key planner-mode-map "\C-c\C-e" 'planner-edit-task-description)
(define-key planner-mode-map "\C-c\C-p" 'planner-task-pending)

(define-key ctl-x-map "pt" 'planner-create-task-from-buffer)
(define-key ctl-x-map "pn" 'remember)
(define-key ctl-x-map "pd" 'planner-diary-add-entry)
(define-key ctl-x-map "pp" 'planner-goto-today)

;; Timeclock
(define-key ctl-x-map "pi" 'planner-timeclock-in)
(define-key ctl-x-map "po" 'timeclock-out)
(define-key ctl-x-map "pc" 'timeclock-change)
(define-key ctl-x-map "pr" 'timeclock-reread-log)
(define-key ctl-x-map "pu" 'timeclock-update-modeline)
(define-key ctl-x-map "pw" 'timeclock-when-to-leave-string)



;;; Basic setup

(setq planner-carry-tasks-forward t
      planner-add-tasks-at-end-flag t
      planner-use-other-window nil)

(setq remember-handler-functions '(remember-planner-append))
(setq remember-annotation-functions planner-annotation-functions)

(setq planner-day-page-template
      "* Tasks\n\n\n* Diary\n\n<lisp>(planner-diary-entries-here)</lisp>\n\n* Schedule\n\n* Notes")
;(planner-insinuate-calendar)
(setq planner-diary-number-of-days 1)
(setq planner-diary-number-of-diary-entries 7)
(setq planner-diary-file diary-file)

;; Set up the appointment system for both task and schedule based appointments
(setq planner-appt-tasks-use-appointments-section-flag t)
(setq planner-appt-sort-schedule-on-update-flag t)
(setq planner-appt-update-appts-on-save-flag t)
(planner-appt-use-tasks-and-schedule)
(planner-appt-schedule-cyclic-insinuate)
(planner-appt-calendar-insinuate)

;; Whenever the planner-mode-hook is run, update any deadlines listed
(add-hook 'planner-mode-hook 'planner-deadline-update)


(defun planner-diary-add-entry (date time text)
  "Prompt for a diary entry to add to `diary-file'.  Will run
planner-annotations to make hyper links"
  (interactive (list (planner-read-date)
  		     (read-string "Time: ")
                     (read-string "Diary entry: ")))
  (save-excursion
    (save-window-excursion
      (make-diary-entry
       (concat
        (let ((cal-date (planner-filename-to-calendar-date date)))
          (if european-calendar-style
              (format "%d/%d/%d"
                      (elt cal-date 1)
                      (elt cal-date 0)
                      (elt cal-date 2))
            (format "%d/%d/%d"
                    (elt cal-date 0)
                    (elt cal-date 1)
                    (elt cal-date 2))))
        " " time " " text " "
        (run-hook-with-args-until-success
         'planner-annotation-functions))))))


;;
;; Add additional support for logging what happened while clocked into
;; a project.
;;
(defvar my-current-task-info nil
  "Full information about the currently clocked in task.")

(defadvice planner-task-in-progress (after planner-timeclock-note activate)
  "Store the full information about the current task on clockin."
  (setq my-current-task-info (planner-current-task-info)))

(defadvice timeclock-out (after planner-timeclock-note activate)
  "Create a note in the for the task describing what happened during
the clocked-in period."
  (when my-current-task-info
    (planner-create-note (goto-task-page my-current-task-info))
    (insert
     "<*P:" (timeclock-seconds-to-string (timeclock-last-period) t)
     "@"
     (format-time-string "%Y-%m-%d %H:%M:%S" (current-time))
     "> "
     (planner-task-description my-current-task-info)
     "\n\n")

    ;; clear the task since we're now logged out
    (setq my-current-task-info nil)
    )
  )

(defun goto-task-page (info)
  "Redirect either to the plan page for the task, or failing that
the day page.  Returns the page selected."
  (let ((plan (planner-task-plan info))
        (date (planner-task-date info)))
    (if plan
        (progn
          (planner-goto-plan-page plan)
          plan)
      (planner-goto date)
      date)))


;;
;; Find all the unmarked "Clocked out" lines and remove the leading asterisk
;; which indicates that the clocked out line has been invoiced.
;;
(defun planner-mark-invoices ()
  "Mark all outstanding clock-outs as currently invoiced."
  (interactive)
  (save-excursion
   (goto-char (point-min))
   (while (re-search-forward "<\\*" nil t)
     (replace-match "<" nil nil))))



;;
;; Finish up by declaring the package defined within.
;;
(provide 'planner-config)
